using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Text;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;
using ServiciosEscritorioSAT.Repositories;
using ServiciosEscritorioSAT.Models;

namespace ServiciosEscritorioSAT.Controllers {
    [ApiController]
    [Route("auth")]
    public class AuthController: ControllerBase {
        private ILogger<AuthController> logger;
        private IHttpContextAccessor httpContextAccessor;

        public AuthController(
            ILogger<AuthController> logger,
            IHttpContextAccessor httpContextAccessor
        ) {
            this.logger = logger;
            this.httpContextAccessor = httpContextAccessor;
        }

        [HttpGet("authenticate/{userName}")]
        public ActionResult<AuthenticatedUser> Authenticate(string userName) {
            UsuariosRepository usuarioRepo = new UsuariosRepository();
            Usuario usuario = usuarioRepo.GetByUserName(userName);
            if (usuario == null) {
                return BadRequest();
            }
            PermisoRepository permisosRepo = new PermisoRepository();
            IEnumerable<Permiso> permisos = permisosRepo.GetByAccessRef(usuario.access_def);
            AuthenticatedUser response = new AuthenticatedUser() {
                access_def = usuario.access_def,
                nombre = usuario.nombre,
                apellidos = usuario.apellidos,
                user_name = usuario.user_name,
                supervisor = usuario.supervisor,
                permisos = permisos.Select(p => p.nombre).ToArray(),
                token = generateJwtToken(usuario)
            };
            AuditLogRepository auditRepo = new AuditLogRepository();
            auditRepo.Create(new AuditLog() {
                action = "login",
                fecha = DateTime.Now,
                access_ref = usuario.access_def,
                entity_name = nameof(Usuario),
                entity_data = usuario.AsText()
            });
            return Ok(response);
        }

        private string generateJwtToken(Usuario user)
        {
            // generate token that is valid for 7 days
            var tokenHandler = new JwtSecurityTokenHandler();
            var key = Encoding.ASCII.GetBytes(
                Environment.GetEnvironmentVariable(Constantes.JWT_SIGN_KEY)
            );
            var tokenDescriptor = new SecurityTokenDescriptor
            {
                Subject = new ClaimsIdentity(new[] { new Claim("id", user.access_def.ToString()) }),
                Expires = DateTime.UtcNow.AddDays(7),
                SigningCredentials = new SigningCredentials(new SymmetricSecurityKey(key), SecurityAlgorithms.HmacSha256Signature)
            };
            var token = tokenHandler.CreateToken(tokenDescriptor);
            return tokenHandler.WriteToken(token);
        }
    }
}