using System;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using ServiciosEscritorioSAT.Models;
using ServiciosEscritorioSAT.Services;
using ServiciosEscritorioSAT.Repositories;
using Microsoft.AspNetCore.Http;

namespace ServiciosEscritorioSAT.Controllers {
    [ApiController]
    [Route("estacionada")]
    public class EstacionadaController : ControllerBase {
        private ILogger<EstacionadaController> logger;
        private IHttpContextAccessor httpContextAccessor;

        public EstacionadaController(
            ILogger<EstacionadaController> logger,
            IHttpContextAccessor httpContextAccessor
        ) {
            this.logger = logger;
            this.httpContextAccessor = httpContextAccessor;
        }

        [Authorize]
        [HttpPost("query")]
        public ActionResult<IEnumerable<Estacionada>> Get([FromBody]ColumnFilter[] filters) {
            EstacionadaRepository repo = new EstacionadaRepository();
            return Ok(repo.Query(filters));
        }

        [Authorize]
        [HttpPost]
        public ActionResult Post(Estacionada datos) {
            EstacionadaRepository repo = new EstacionadaRepository();
            Estacionada est = repo.Create(datos);
            AuditLogRepository auditRepo = new AuditLogRepository();
            auditRepo.Create(new AuditLog() {
                action = "insert",
                fecha = DateTime.Now,
                access_ref = GetUser().access_def,
                entity_name = nameof(Estacionada),
                entity_data = est.AsText()
            });
            MessageSender.Send(new Message() {
                origin=MessageOriginDestination.SERVIDOR,
                destination=MessageOriginDestination.TODOS,
                action=MessageAction.INSERT,
                content=est.AsText(),
                entity=nameof(Estacionada)
            });
            return Ok();
        }

        [Authorize]
        [HttpPut("{id:int}")]
        public ActionResult Put(int id, Estacionada datos) {
            EstacionadaRepository repo = new EstacionadaRepository();
            Estacionada est = repo.Update(id, datos);
            AuditLogRepository auditRepo = new AuditLogRepository();
            if (est != null) {
                auditRepo.Create(new AuditLog() {
                    action = "update",
                    fecha = DateTime.Now,
                    access_ref = GetUser().access_def,
                    entity_name = nameof(Estacionada),
                    entity_data = est.AsText()
                });
                MessageSender.Send(new Message() {
                    origin=MessageOriginDestination.SERVIDOR,
                    destination=MessageOriginDestination.TODOS,
                    action=MessageAction.UPDATE,
                    content=datos.AsText(),
                    entity=nameof(Estacionada)
                });
                return Ok();
            } else {
                return NotFound();
            }
        }

        [Authorize]
        [HttpDelete("{id:int}")]
        public ActionResult Delete(int id) {
            EstacionadaRepository repo = new EstacionadaRepository();
            Estacionada est = repo.Delete(id);
            AuditLogRepository auditRepo = new AuditLogRepository();
            if (est != null) {
                auditRepo.Create(new AuditLog() {
                    action = "delete",
                    fecha = DateTime.Now,
                    access_ref = GetUser().access_def,
                    entity_name = nameof(Estacionada),
                    entity_data = est.AsText()
                });
                MessageSender.Send(new Message() {
                    origin=MessageOriginDestination.SERVIDOR,
                    destination=MessageOriginDestination.TODOS,
                    action=MessageAction.DELETE,
                    content=est.AsText(),
                    entity=nameof(Estacionada)
                });
                return Ok();
            } else {
                return NotFound();
            }
        }

        private Usuario GetUser() {
            return (Usuario)httpContextAccessor.HttpContext.Items["User"];
        }
    }
}
