using System;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using ServiciosEscritorioSAT.Models;
using ServiciosEscritorioSAT.Services;
using ServiciosEscritorioSAT.Repositories;
using Microsoft.AspNetCore.Http;

namespace ServiciosEscritorioSAT.Controllers {
    [ApiController]
    [Route("estacionada_historico")]
    public class EstacionadaHistoricoController : ControllerBase {
        private ILogger<EstacionadaHistoricoController> logger;
        private IHttpContextAccessor httpContextAccessor;

        public EstacionadaHistoricoController(
            ILogger<EstacionadaHistoricoController> logger,
            IHttpContextAccessor httpContextAccessor
        ) {
            this.logger = logger;
            this.httpContextAccessor = httpContextAccessor;
        }

        [Authorize]
        [HttpPost]
        public ActionResult Post(EstacionadaHistorico datos) {
            EstacionadaHistoricoRepository repo = new EstacionadaHistoricoRepository();
            EstacionadaHistorico hist = repo.Create(datos);
            AuditLogRepository auditRepo = new AuditLogRepository();
            auditRepo.Create(new AuditLog() {
                action = "insert",
                fecha = DateTime.Now,
                access_ref = GetUser().access_def,
                entity_name = nameof(EstacionadaHistorico),
                entity_data = hist.AsText()
            });
            EstacionadaRepository estRepo = new EstacionadaRepository();
            Estacionada est = estRepo.QueryById(hist.estacionada_id);
            MessageSender.Send(new Message() {
                origin=MessageOriginDestination.SERVIDOR,
                destination=MessageOriginDestination.TODOS,
                action=MessageAction.DELETE,
                content=est.AsText(),
                entity=nameof(Estacionada)
            });
            return Ok();
        }

        private Usuario GetUser() {
            return (Usuario)httpContextAccessor.HttpContext.Items["User"];
        }
    }
}