using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.SqlClient;
using RepoDb;
using ServiciosEscritorioSAT.Models;
using ServiciosEscritorioSAT.Helpers;

namespace ServiciosEscritorioSAT.Repositories {
    public class EstacionadaRepository {
        public IEnumerable<Estacionada> Query(ColumnFilter[] filters) {
            using (var connection = new SqlConnection(Environment.GetEnvironmentVariable(Constantes.CONNECTION_STRING))) {
                if (filters != null && filters.Count() > 0) {
                    return connection.Query<Estacionada>(QueryHelper.BuildQueryFields(filters)).ToArray();
                } else {
                    return connection.ExecuteQuery<Estacionada>("select * from estacionadas where not exists (select 1 from estacionadas_historicos where estacionadas_historicos.estacionada_id = estacionadas.id)").ToArray();
                }
            }
        }

        public Estacionada QueryById(int id) {
            using (var connection = new SqlConnection(Environment.GetEnvironmentVariable(Constantes.CONNECTION_STRING))) {
                return connection.Query<Estacionada>(id).FirstOrDefault();
            }
        }

        public Estacionada Create(Estacionada data) {
            using (var connection = new SqlConnection(Environment.GetEnvironmentVariable(Constantes.CONNECTION_STRING))) {
                var id = connection.Insert<Estacionada>(data);
                Estacionada est = connection.Query<Estacionada>(id).First();
                return est;
            }
        }

        public Estacionada Update(int id, Estacionada data) {
            using (var connection = new SqlConnection(Environment.GetEnvironmentVariable(Constantes.CONNECTION_STRING))) {
                var updatedRows = connection.Update<Estacionada>(data, id);
                if (updatedRows == 1) {
                    return data;
                } else {
                    return null;
                }
            }
        }

        public Estacionada Delete(int id) {
            using (var connection = new SqlConnection(Environment.GetEnvironmentVariable(Constantes.CONNECTION_STRING))) {
                var estacionada = connection.Query<Estacionada>(id).FirstOrDefault();
                var deletedRows = connection.Delete<Estacionada>(id);
                return estacionada;
            }
        }
    }
}