import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { HttpClientModule, HTTP_INTERCEPTORS } from "@angular/common/http";
import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { StoreModule } from '@ngrx/store';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { EffectsModule } from "@ngrx/effects";
import { appReducers } from './app.reducers';
import {StoreDevtoolsModule} from "@ngrx/store-devtools";
import {StoreRouterConnectingModule} from "@ngrx/router-store";
import {environment} from "src/environments/environment";
import  * as DesktopEffects from "./state/effects";
import { ReactiveFormsModule } from '@angular/forms';
import { InjectableRxStompRpcConfig, RxStompService, rxStompServiceFactory } from '@stomp/ng2-stompjs';
import { myRxStompConfig } from './my-rx-stomp-config';

// Angular Material

import {MatToolbarModule} from "@angular/material/toolbar";
import {MatIconModule} from "@angular/material/icon";
import {MatCardModule} from "@angular/material/card";
import {MatButtonModule} from "@angular/material/button";
import {MatSidenavModule} from '@angular/material/sidenav'; 
import {MatBadgeModule} from '@angular/material/badge';

// Desktop components

import { MainMenuComponent } from './components/main-menu/main-menu.component';
import { MainNotificacionesComponent } from './components/main-notificaciones/main-notificaciones.component';
import { MainDesktopComponent } from './components/main-desktop/main-desktop.component';

// Feature modules

import { EstacionadasModule } from './modulos/estacionadas/estacionadas.module';
import { MainNotificacionesContainerComponent } from './components/main-notificaciones-container/main-notificaciones-container.component';
import { AuthenticationComponent } from './components/authentication/authentication.component';
import { AuthenticatedInterceptor } from './common/services/authenticated.interceptor';
import { NoAuthComponent } from './components/no-auth/no-auth.component';
import { LogoutComponent } from './components/logout/logout.component';


@NgModule({
  declarations: [
    AppComponent,
    MainMenuComponent,
    MainNotificacionesComponent,
    MainDesktopComponent,
    MainNotificacionesContainerComponent,
    AuthenticationComponent,
    NoAuthComponent,
    LogoutComponent
  ],
  imports: [
    BrowserModule,
    AppRoutingModule,
    ReactiveFormsModule,
    StoreModule.forRoot(appReducers, {
      runtimeChecks: {
        strictStateImmutability: false,
        strictActionImmutability: false
      }
    }),
    StoreRouterConnectingModule.forRoot({stateKey: 'router'}),
    BrowserAnimationsModule,
    EffectsModule.forRoot([DesktopEffects.DesktopEffects]),
    HttpClientModule,
    EstacionadasModule,
    MatToolbarModule,
    MatIconModule,
    MatCardModule,
    MatButtonModule,
    MatSidenavModule,
    MatBadgeModule,
    StoreDevtoolsModule.instrument({
      maxAge: 25, // Retains last 25 states
      logOnly: environment.production
    })
  ],
  providers: [
    {
      provide: InjectableRxStompRpcConfig,
      useValue: myRxStompConfig
    },
    {
      provide: RxStompService,
      useFactory: rxStompServiceFactory,
      deps: [InjectableRxStompRpcConfig]
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: AuthenticatedInterceptor,
      multi: true
    }
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }
