// Adaptado de https://stomp-js.github.io/guide/ng2-stompjs/ng2-stomp-with-angular7.html

import { Injectable } from '@angular/core';
import { RxStompService } from '@stomp/ng2-stompjs';
import { Subject, Subscription } from 'rxjs';
import { Message } from '../messages/message';

@Injectable({
  providedIn: 'root'
})
export class MessagesService {

  static TOPIC = "/topic/escritorio-sat";

  private stompSubscription!: Subscription;
  private subject$ = new Subject<Message>();
  public messages$ = this.subject$.asObservable();

  constructor(
    private rxStompService: RxStompService
  ) {
    this.stompSubscription = rxStompService.watch({destination:MessagesService.TOPIC}).subscribe(data => {
      const dataObj = JSON.parse(data.body);
      if (this.isMessage(dataObj)) {
        this.subject$.next(dataObj);
      }
    })
  }

  send(msg: Message): void {
    this.rxStompService.publish({
      destination: MessagesService.TOPIC,
      body: JSON.stringify(msg)
    });
  }

  disconnect(): void {
    this.stompSubscription.unsubscribe();
    this.subject$.complete();
  }

  private isMessage(data: any): data is Message {
    return data && data.origin && data.content;
  }
}
