import { Component, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import Estacionada from '../../models/estacionada';
import { FormularioEstacionadaComponent } from '../formulario-estacionada/formulario-estacionada.component';
import * as estacionadasActions from '../../state/actions';
import { FormularioAsignacionEstacionadasComponent } from '../formulario-asignacion-estacionadas/formulario-asignacion-estacionadas.component';
import { FormularioRealizacionEstacionadaComponent } from '../formulario-realizacion-estacionada/formulario-realizacion-estacionada.component';
import EstacionadaHistorico from '../../models/estacionada-historico';
import { AuthenticationService } from 'src/app/common/services/authentication.service';
import AuthenticatedUser from 'src/app/common/models/authenticated-user';

@Component({
  selector: 'esat-botonera-acciones-principal',
  templateUrl: './botonera-acciones-principal.component.html',
  styleUrls: ['./botonera-acciones-principal.component.scss']
})
export class BotoneraAccionesPrincipalComponent implements OnInit {

  filaSeleccionada: Estacionada | null = null;
  usuarioActual: AuthenticatedUser | null = null;

  constructor(
    private dialog: MatDialog,
    private store: Store<AppState>,
    private authenticationSvc: AuthenticationService
  ) { }

  ngOnInit(): void {
    this.store.select("estacionadas").subscribe(datos => {
      this.filaSeleccionada = datos.filaSeleccionada;
    });
    this.usuarioActual = this.authenticationSvc.currentUser;
  }

  nuevaEstacionada(): void {
    const dialogRef = this.dialog.open(FormularioEstacionadaComponent, {
      data: {estacionada: null}
    });
    dialogRef.afterClosed().subscribe(estacionada => {
      if (estacionada) {
        this.store.dispatch(estacionadasActions.crearEstacionada({estacionada:estacionada}));
      }
    });
  }

  editarEstacionada(): void {
    const dialogRef = this.dialog.open(FormularioEstacionadaComponent, {
      data: {estacionada: this.filaSeleccionada}
    });
    dialogRef.afterClosed().subscribe(estacionada => {
      if (estacionada) {
        this.store.dispatch(estacionadasActions.actualizarEstacionada({estacionada:estacionada}));
      }
    });
  }

  asignarEstacionadas(): void {
    const dialogRef = this.dialog.open(FormularioAsignacionEstacionadasComponent);
    dialogRef.afterClosed().subscribe(usuario => {
      if (usuario) {
        const nueva:Estacionada = {
          ...this.filaSeleccionada as Estacionada,
          access_ref: usuario.access_def
        };
        this.store.dispatch(estacionadasActions.actualizarEstacionada({estacionada: nueva}));
      }
    })
  }

  desasignarEstacionadas(): void {
    if (confirm("¿Confirma que desea eliminar la asignación de las filas seleccionadas?")) {
      const nueva: Estacionada = {
        ...this.filaSeleccionada as Estacionada,
        access_ref: null
      };
      this.store.dispatch(estacionadasActions.actualizarEstacionada({estacionada: nueva}));
    }
  }

  eliminarEstacionada(): void {
    if (confirm('¿Confirma que desea eliminar la fila seleccionada?')) {
      this.store.dispatch(estacionadasActions.eliminarEstacionada({estacionada:this.filaSeleccionada as Estacionada}));
    }
  }

  realizarEstacionada(): void {
    const dialogRef = this.dialog.open(FormularioRealizacionEstacionadaComponent);
    dialogRef.afterClosed().subscribe(motivo => {
      if (motivo) {
        const historico:EstacionadaHistorico = {
          estacionada_id: this.filaSeleccionada?.id as number,
          access_ref: this.authenticationSvc.currentUser?.access_def as number,
          fecha: new Date(),
          estacionadas_motivos_id: motivo.id
        };
        this.store.dispatch(estacionadasActions.realizarEstacionada({estacionada_historico: historico}));
      }
    })
  }

}
