import { Component, OnInit } from '@angular/core';
import { DomSanitizer, SafeUrl } from '@angular/platform-browser';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import Usuario from 'src/app/common/models/usuario';
import Estacionada from '../../models/estacionada';
import Tipologia from '../../models/tipologia';

@Component({
  selector: 'esat-detalle-fila',
  templateUrl: './detalle-fila.component.html',
  styleUrls: ['./detalle-fila.component.scss']
})
export class DetalleFilaComponent implements OnInit {

  filaSeleccionada: Estacionada | null = null;
  tipologias: Tipologia[] = [];
  usuarios: Usuario[] = [];

  dwelling_url: SafeUrl = "";
  call_url: SafeUrl = "";

  constructor(
    private store: Store<AppState>,
    private domSanitizer: DomSanitizer
  ) { }

  ngOnInit(): void {
    this.store.select("estacionadas").subscribe(datos => {
      this.filaSeleccionada = datos.filaSeleccionada;
      this.tipologias = datos.tipologias;
      this.dwelling_url = this.generarDwellingUrl();
      this.call_url = this.generarCallUrl();
    });
    this.store.select("desktop").subscribe(datos => {
      this.usuarios = datos.usuarios;
    })
  }

  obtenerNombreTipologia(id: number): string {
    const tipologia = this.tipologias.find(x => x.id === id);
    if (tipologia) {
      return tipologia.nombre;
    }
    return '';
  }

  obtenerNombreUsuario(id: number|null): string {
    if (id) {
      const usuario = this.usuarios.find(x => x.access_def === id);
      if (usuario) {
        return usuario.user_name;
      }
    }
    return '';
  }

  private generarDwellingUrl(): SafeUrl {
    return this.domSanitizer.bypassSecurityTrustUrl(`pnc:dwelling/${this.filaSeleccionada?.equip_id}`);
  }

  private generarCallUrl(): SafeUrl {
    return this.domSanitizer.bypassSecurityTrustUrl(`pnc:dwelling/${this.filaSeleccionada?.equip_id}/call/${this.filaSeleccionada?.telefono}`);
  }

}
