import { Component, EventEmitter, OnInit, Output } from '@angular/core';
import { FormControl } from '@angular/forms';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import { ColumnFilter } from 'src/app/common/models/column-filter';
import FilterOperation from 'src/app/common/models/filter-operation';
import Usuario from 'src/app/common/models/usuario';
import Tipologia from '../../models/tipologia';

@Component({
  selector: 'tr [esat-filtros-estacionadas]',
  templateUrl: './filtros-estacionadas.component.html',
  styleUrls: ['./filtros-estacionadas.component.scss']
})
export class FiltrosEstacionadasComponent implements OnInit {

  @Output() onSearch = new EventEmitter<ColumnFilter[]>();

  centrales = ['MA', 'S2'];
  tipologias: Tipologia[] = [];
  usuarios: Usuario[] = [];

  equip_id = new FormControl();
  fecha_activacion = new FormControl();
  access_ref = new FormControl();
  nombre = new FormControl();
  apellidos = new FormControl();
  central = new FormControl();
  tipologia = new FormControl();

  constructor(private store: Store<AppState>) { }

  ngOnInit(): void {
    this.store.select("estacionadas").subscribe(datos => {
      this.tipologias = datos.tipologias;
    });
    this.store.select("desktop").subscribe(datos => {
      this.usuarios = datos.usuarios;
    })
  }

  filtrar(): void {
    this.onSearch.emit(this.construirFiltros());
  }

  construirFiltros(): ColumnFilter[] {
    const filtros: ColumnFilter[] = [];
    if (this.equip_id.value && this.equip_id.value.length > 0) {
      filtros.push({
        columnName: 'equip_id',
        filterOperation: FilterOperation.LIKE,
        filterValue: this.equip_id.value
      });
    }
    if (this.tipologia.value) {
      filtros.push({
        columnName: 'tipologia_id',
        filterOperation: FilterOperation.EQUAL,
        filterValue: this.tipologia.value
      });
    }
    if (this.access_ref.value) {
      filtros.push({
        columnName: 'access_ref',
        filterOperation: FilterOperation.EQUAL,
        filterValue: this.access_ref.value
      });
    }
    if (this.nombre.value && this.nombre.value.length > 0) {
      filtros.push({
        columnName: 'nombre',
        filterOperation: FilterOperation.LIKE,
        filterValue: this.nombre.value
      });
    }
    if (this.apellidos.value && this.apellidos.value.length > 0) {
      filtros.push({
        columnName: 'apellidos',
        filterOperation: FilterOperation.LIKE,
        filterValue: this.apellidos.value
      });
    }
    if (this.central.value) {
      filtros.push({
        columnName: 'central',
        filterOperation: FilterOperation.EQUAL,
        filterValue: this.central.value
      });
    }
    return filtros;
  }
}
