import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormControl, Validators } from '@angular/forms';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import Estacionada from '../../models/estacionada';
import Tipologia from '../../models/tipologia';

@Component({
  selector: 'esat-formulario-estacionada',
  templateUrl: './formulario-estacionada.component.html',
  styleUrls: ['./formulario-estacionada.component.scss']
})
export class FormularioEstacionadaComponent implements OnInit {

  tipologias: Tipologia[] = [];
  centrales: string[] = ['MA', 'S2'];

  estacionadaForm = this.formBuilder.group({
    equip_id: new FormControl('', [Validators.required]),
    nombre: new FormControl('', [Validators.required]),
    apellidos: new FormControl('', [Validators.required]),
    telefono: new FormControl('', [Validators.required]),
    tipologia_id: new FormControl('', [Validators.required]),
    fecha_inicio: new FormControl('', [Validators.required]),
    fecha_fin: new FormControl(''),
    hora_activacion: new FormControl('', [Validators.required]),
    central: new FormControl('', [Validators.required]),
    observaciones: new FormControl('')
  });

  constructor(
    private store: Store<AppState>,
    private dialogRef: MatDialogRef<FormularioEstacionadaComponent>,
    private formBuilder: FormBuilder,
    @Inject(MAT_DIALOG_DATA) public data: {estacionada: Estacionada | null}
  ) { }

  ngOnInit(): void {
    this.store.select("estacionadas").subscribe(estacionadas => {
      this.tipologias = estacionadas.tipologias;
    });
    if (this.data.estacionada) {
      this.estacionadaForm.get('equip_id')?.setValue(this.data.estacionada.equip_id);
      this.estacionadaForm.get('nombre')?.setValue(this.data.estacionada.nombre);
      this.estacionadaForm.get('apellidos')?.setValue(this.data.estacionada.apellidos);
      this.estacionadaForm.get('telefono')?.setValue(this.data.estacionada.telefono);
      this.estacionadaForm.get('tipologia_id')?.setValue(this.data.estacionada.tipologia_id);
      this.estacionadaForm.get('fecha_inicio')?.setValue(this.data.estacionada.fecha_inicio);
      this.estacionadaForm.get('fecha_fin')?.setValue(this.data.estacionada.fecha_fin);
      this.estacionadaForm.get('hora_activacion')?.setValue(this.data.estacionada.hora_activacion);
      this.estacionadaForm.get('central')?.setValue(this.data.estacionada.central);
      this.estacionadaForm.get('observaciones')?.setValue(this.data.estacionada.observaciones);
    }
  }

  cancelar(): void {
    this.dialogRef.close();
  }

  guardar(): void {
    const estacionada: Estacionada = {
      equip_id: this.estacionadaForm.get('equip_id')?.value,
      fecha_inicio: this.estacionadaForm.get('fecha_inicio')?.value,
      nombre: this.estacionadaForm.get('nombre')?.value,
      apellidos: this.estacionadaForm.get('apellidos')?.value,
      telefono: this.estacionadaForm.get('telefono')?.value,
      tipologia_id: this.estacionadaForm.get('tipologia_id')?.value,
      fecha_fin: null,
      central: this.estacionadaForm.get('central')?.value,
      periodicidad_semanal: '',
      access_ref: this.data.estacionada?.access_ref || null,
      observaciones: this.estacionadaForm.get('observaciones')?.value,
      hora_activacion: this.estacionadaForm.get('hora_activacion')?.value,
      id: this.data.estacionada?.id || null
    };
    this.dialogRef.close(estacionada);
  }
}
