import { Component, OnDestroy, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import AuthenticatedUser from 'src/app/common/models/authenticated-user';
import Usuario from 'src/app/common/models/usuario';
import Estacionada from '../../models/estacionada';
import Tipologia from '../../models/tipologia';
import * as estacionadasActions from '../../state/actions';
import * as desktopActions from '../../../../state/actions';
import { ColumnFilter } from 'src/app/common/models/column-filter';
import { AuthenticationService } from 'src/app/common/services/authentication.service';

@Component({
  selector: 'esat-tabla-estacionadas',
  templateUrl: './tabla-estacionadas.component.html',
  styleUrls: ['./tabla-estacionadas.component.scss']
})
export class TablaEstacionadasComponent implements OnInit, OnDestroy {

  private activacionWorker: Worker | undefined;

  datos: Estacionada[] = [];
  filaSeleccionada: Estacionada | null = null;
  tipologias: Tipologia[] = [];
  usuarios: Usuario[] = [];
  usuarioActual: AuthenticatedUser | null = null;

  constructor(
    private store: Store<AppState>,
    private authenticationSvc: AuthenticationService
  ) { }

  ngOnInit(): void {
    this.usuarioActual = this.authenticationSvc.currentUser;
    if (typeof Worker !== 'undefined') {
      this.activacionWorker = new Worker('../../workers/activacion.worker', {type:'module'});
      this.activacionWorker.onmessage = this.notificarActivacion.bind(this);
    }
    this.store.select("desktop").subscribe(desktop => {
      this.usuarios = desktop.usuarios;
    });
    this.store.select("estacionadas").subscribe(estacionadas => {
      this.datos = estacionadas.estacionadas;
      this.filaSeleccionada = estacionadas.filaSeleccionada;
      this.tipologias = estacionadas.tipologias;
      this.activacionWorker?.postMessage(estacionadas.estacionadas.filter(x => x.access_ref === this.usuarioActual?.access_def));
    });
    this.store.dispatch(estacionadasActions.cargarEstacionadas({filtros:[]}));
  }

  ngOnDestroy(): void {
    if (this.activacionWorker) {
      this.activacionWorker.terminate();
    }
  }

  onFilaSeleccionada(fila: Estacionada): void {
    this.store.dispatch(estacionadasActions.seleccionarEstacionada({estacionada: fila}));
  }

  onSearch(filtros: ColumnFilter[]) {
    this.store.dispatch(estacionadasActions.cargarEstacionadas({filtros:filtros}));
  }

  private notificarActivacion(ev: any): void {
    const estacionada = ev.data as Estacionada;
    this.store.dispatch(desktopActions.addNotification({text:`La estacionada de la vivienda ${estacionada.equip_id} se va a activar próximamente`}));
  }

}
