import { Component, OnDestroy, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import MessageAction from 'src/app/common/messages/message-action';
import { MessagesService } from 'src/app/common/services/messages.service';
import Estacionada from './models/estacionada';
import * as estacionadasActions from './state/actions';

@Component({
  selector: 'esat-estacionadas',
  templateUrl: './estacionadas.component.html',
  styleUrls: ['./estacionadas.component.scss']
})
export class EstacionadasComponent implements OnInit, OnDestroy {

  constructor(
    private store:Store<AppState>,
    private messageSvc: MessagesService
  ) { }

  ngOnInit(): void {
    this.store.dispatch(estacionadasActions.cargarTipologias());
    this.store.dispatch(estacionadasActions.cargarEstacionadasMotivos());
    this.messageSvc.messages$.subscribe(m => {
      if (m.entity === 'Estacionada') {
        const estacionada:Estacionada = JSON.parse(m.content);
        if (m.action === MessageAction.INSERT) {
          this.store.dispatch(estacionadasActions.mensajeInsercionEstacionada({estacionada: estacionada}));
        } else if (m.action === MessageAction.UPDATE) {
          this.store.dispatch(estacionadasActions.mensajeActualizacionEstacionada({estacionada:estacionada}));
        } else if (m.action === MessageAction.DELETE) {
          this.store.dispatch(estacionadasActions.mensajeEliminacionEstacionada({estacionada:estacionada}));
        }
      }
    });
  }

  ngOnDestroy(): void {
    this.messageSvc.disconnect();
  }

}
