import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { ColumnFilter } from 'src/app/common/models/column-filter';
import Estacionada from '../models/estacionada';

@Injectable({
  providedIn: 'root'
})
export class EstacionadasService {

  static SERVICE_URL = "/servicios/estacionada";
  
  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  }

  constructor(private http:HttpClient) { }

  get(filtros:ColumnFilter[]): Observable<Estacionada[]> {
    return this.http.post<Estacionada[]>(
      `${EstacionadasService.SERVICE_URL}/query`,
      filtros,
      this.httpOptions
    );
  }

  create(estacionada: Estacionada): Observable<Estacionada> {
    return this.http.post<Estacionada>(
      EstacionadasService.SERVICE_URL,
      estacionada,
      this.httpOptions
    );
  }

  update(estacionada: Estacionada): Observable<Estacionada> {
    return this.http.put<Estacionada>(
      `${EstacionadasService.SERVICE_URL}/${estacionada.id}`,
      estacionada,
      this.httpOptions
    );
  }

  delete(estacionada: Estacionada): Observable<Estacionada> {
    return this.http.delete<Estacionada>(
      `${EstacionadasService.SERVICE_URL}/${estacionada.id}`,
      this.httpOptions
    )
  }
}
