import { Injectable } from "@angular/core";
import { Actions, createEffect, ofType } from "@ngrx/effects";
import { EMPTY, empty, of } from "rxjs";
import { catchError, map, mergeMap } from "rxjs/operators";
import Estacionada from "../../models/estacionada";
import Tipologia from "../../models/tipologia";
import { EstacionadasHistoricosService } from "../../services/estacionadas-historicos.service";
import { EstacionadasMotivosService } from "../../services/estacionadas-motivos.service";
import { EstacionadasService } from "../../services/estacionadas.service";
import { TipologiasService } from "../../services/tipologias.service";
import * as estacionadasActions from "../actions";


@Injectable()
export class EstacionadasEffects {

    cargarEstacionadas = createEffect(() => 
        this.action$.pipe(
            ofType(estacionadasActions.cargarEstacionadas),
            mergeMap(({filtros}) =>
                this.estacionadasSvc.get(filtros).pipe(
                    map((estacionadas: Estacionada[]) => 
                        estacionadasActions.cargarEstacionadasOk({estacionadas})
                    ),
                    catchError((error) => of(estacionadasActions.cargarEstacionadasError({error:error})))
                )
            )
        )
    );

    cargarTipologias = createEffect(() =>
        this.action$.pipe(
            ofType(estacionadasActions.cargarTipologias),
            mergeMap(()=>
                this.tipologiasSvc.getAll().pipe(
                    map((tipologias:Tipologia[]) =>
                        estacionadasActions.cargarTipologiasOk({tipologias})
                    ),
                    catchError((error) => of(estacionadasActions.cargarTipologiasError({error:error})))
                )
            )
        )
    );

    crearEstacionada = createEffect(() => 
        this.action$.pipe(
            ofType(estacionadasActions.crearEstacionada),
            mergeMap(({estacionada}) =>
                this.estacionadasSvc.create(estacionada).pipe(
                    map((estacionada: Estacionada) =>
                        estacionadasActions.crearEstacionadaOk()
                    ),
                    catchError((error) => of(estacionadasActions.cargarEstacionadasError({error: error})))
                )
            )
        )
    );

    actualizarEstacionada = createEffect(() =>
        this.action$.pipe(
            ofType(estacionadasActions.actualizarEstacionada),
            mergeMap(({estacionada}) =>
                this.estacionadasSvc.update(estacionada).pipe(
                    map(() =>
                        estacionadasActions.actualizarEstacionadaOk()
                    ),
                    catchError((error) => of(estacionadasActions.actualizarEstacionadaError({error: error})))
                )
            )
        )
    );

    eliminarEstacionada = createEffect(() =>
        this.action$.pipe(
            ofType(estacionadasActions.eliminarEstacionada),
            mergeMap(({estacionada}) =>
                this.estacionadasSvc.delete(estacionada).pipe(
                    map(() =>
                        estacionadasActions.eliminarEstacionadaOk()
                    ),
                    catchError((error) => of(estacionadasActions.eliminarEstacionadaError({error:error})))
                )
            )
        )
    );

    cargarEstacionadasMotivos = createEffect(() =>
        this.action$.pipe(
            ofType(estacionadasActions.cargarEstacionadasMotivos),
            mergeMap(() =>
                this.estacionadasMotivosSvc.get().pipe(
                    map((motivos) =>
                        estacionadasActions.cargarEstacionadasMotivosOk({motivos:motivos})
                    ),
                    catchError((error) => of(estacionadasActions.cargarEstacionadasMotivosError({error: error})))
                )
            )
        )
    )

    realizarEstacionada = createEffect(() =>
        this.action$.pipe(
            ofType(estacionadasActions.realizarEstacionada),
            mergeMap(({estacionada_historico}) =>
                this.estacionadasHistoricosSvc.create(estacionada_historico).pipe(
                    map((_) => EMPTY)
                )
            )
        ),
        {dispatch: false}
    )

    constructor(
        private action$: Actions,
        private estacionadasSvc: EstacionadasService,
        private tipologiasSvc: TipologiasService,
        private estacionadasMotivosSvc: EstacionadasMotivosService,
        private estacionadasHistoricosSvc: EstacionadasHistoricosService
    ) { }
}