import { createReducer, on } from "@ngrx/store";
import Estacionada from "../../models/estacionada";
import {Action} from "@ngrx/store";
import * as estacionadasActions from "../actions";
import Tipologia from "../../models/tipologia";
import EstacionadaMotivo from "../../models/estacionada-motivo";

export interface EstacionadasState {
    tipologias: Tipologia[],
    estacionadas: Estacionada[],
    filaSeleccionada: Estacionada | null,
    motivos: EstacionadaMotivo[]
}

export const estacionadaInitialState: EstacionadasState = {
    tipologias: [],
    estacionadas: [],
    filaSeleccionada: null,
    motivos: []
}

const _estacionadasReducers = createReducer(
    estacionadaInitialState,
    on(estacionadasActions.cargarEstacionadasOk,
        (state, {estacionadas}): EstacionadasState => ({
            ...state,
            estacionadas:estacionadas
        })
    ),
    on(estacionadasActions.seleccionarEstacionada,
        (state, {estacionada}): EstacionadasState => ({
            ...state,
            filaSeleccionada: estacionada
        })
    ),
    on(estacionadasActions.cargarTipologiasOk,
        (state, {tipologias}): EstacionadasState => ({
            ...state,
            tipologias: tipologias
        })
    ),
    on(estacionadasActions.mensajeInsercionEstacionada,
        (state, {estacionada}): EstacionadasState => ({
            ...state,
            estacionadas: [...state.estacionadas, estacionada]
        })
    ),
    on(estacionadasActions.mensajeActualizacionEstacionada,
        (state, {estacionada}): EstacionadasState => ({
            ...state,
            estacionadas: state.estacionadas.map(x => {
                if (x.id === estacionada.id) {
                    return estacionada;
                } else {
                    return x;
                }
            }),
            filaSeleccionada: (state.filaSeleccionada && state.filaSeleccionada.id === estacionada.id) 
                ? estacionada 
                : (state.filaSeleccionada 
                    ? state.filaSeleccionada 
                    : null)
        })
    ),
    on(estacionadasActions.mensajeEliminacionEstacionada,
        (state, {estacionada}): EstacionadasState => ({
            ...state,
            estacionadas: state.estacionadas.filter(x => x.id !== estacionada.id)
        })
    ),
    on(estacionadasActions.cargarEstacionadasMotivosOk,
        (state, {motivos}): EstacionadasState => ({
            ...state,
            motivos: motivos
        })
    )
)

export function estacionadasReducers(state: EstacionadasState | undefined, action: Action) {
    return _estacionadasReducers(state, action);
}