import { Action, createReducer, on } from "@ngrx/store";
import Notification from "src/app/common/models/notification";
import Usuario from "src/app/common/models/usuario";
import IApplication from "../../common/models/iapplication";
import * as desktopActions from "../actions";

export interface DesktopState {
    registeredApps: IApplication[]
    error: string | null
    currentApp: IApplication | null,
    notificacionesOpen: boolean,
    usuarios: Usuario[],
    notificaciones: Notification[]
}

export const desktopInitialState: DesktopState = {
    registeredApps: [],
    error: null,
    currentApp: null,
    notificacionesOpen: false,
    usuarios: [],
    notificaciones: []
}

const _desktopReducers = createReducer(
    desktopInitialState,
    on(desktopActions.registerApplicationOk,
        (state, {app}): DesktopState => ({
            ...state,
            registeredApps: [...state.registeredApps, app],
            error: null
        })
    ),
    on(desktopActions.openApplicationOk,
        (state, {app}): DesktopState => ({
            ...state,
            currentApp: app
        })
    ),
    on(desktopActions.openDesktopOk, 
        (state): DesktopState => ({
            ...state,
            currentApp: null
        })
    ),
    on(desktopActions.toggleNotificacionesContainer,
        (state): DesktopState => ({
            ...state,
            notificacionesOpen: !state.notificacionesOpen
        })
    ),
    on(desktopActions.loadUsersOk,
        (state, {users}): DesktopState => ({
            ...state,
            usuarios: users
        })
    ),
    on(desktopActions.addNotification,
        (state, {text}): DesktopState => ({
            ...state,
            notificaciones: [{
                text: text,
                read: false
            }, ...state.notificaciones]
        })
    ),
    on(desktopActions.readNotification,
        (state, {notification}): DesktopState => ({
            ...state,
            notificaciones: state.notificaciones.map(x => {
                if (x === notification) {
                    return {text: x.text, read: true}
                } else {
                    return x;
                }
            })
        }))
);

export function desktopReducers(state: DesktopState | undefined, action: Action) {
    return _desktopReducers(state, action);
}