drop database if exists escritorio_sat;
go
create database escritorio_sat;
go
use escritorio_sat;
go

create table usuarios (
    access_def int identity,
    user_name nvarchar(50) not null,
    nombre nvarchar(100) not null,
    apellidos nvarchar(200) not NULL,
    central char(2) NOT NULL,
    supervisor bit not null,
    
    constraint pk_usuarios primary key (access_def)
);

create table permisos (
    id int identity,
    access_ref int not null,
    nombre varchar(30) not null
    
    constraint pk_permisos primary key (id),
    constraint fk_permisos_usuarios foreign key (access_ref) references usuarios(access_def)
);

create table tipologias (
    id int identity,
    nombre nvarchar(20) not null
  
    constraint pk_tipologias primary key (id)
);

create table estacionadas (
    id int identity,
    equip_id varchar(12) not null,
    fecha_inicio datetime not null,
    fecha_fin datetime,
    hora_activacion char(5) not null,
    nombre nvarchar(100) not null,
    apellidos nvarchar(200) not null,
    access_ref int,
    central varchar(2) not null,
    tipologia_id int not null,
    telefono varchar(20) not null,
    periodicidad_semanal char(7) not null,
    observaciones nvarchar(250),
    
    constraint pk_estacionadas primary key (id),
    constraint fk_estacionadas_tipologias foreign key (tipologia_id) references tipologias(id),
    constraint fk_estacionadas_usuarios foreign key (access_ref) references usuarios(access_def)
);

create table estacionadas_motivos (
    id int identity,
    nombre varchar(50) not null,
    
    constraint pk_estacionadas_motivos primary key (id)
);

create table estacionadas_historicos (
    id int identity,
    fecha datetime not null,
    access_ref int not null,
    estacionada_id int not null,
    estacionadas_motivos_id int not null,
    
    constraint pk_estacionadas_historico primary key (id),
    constraint fk_estacionadas_historico_usuarios foreign key (access_ref) references usuarios(access_def),
    constraint fk_estacionadas_historico_estacionadas foreign key (estacionada_id) references estacionadas(id),
    constraint fk_estacionadas_historico_estacionadas_motivos foreign key (estacionadas_motivos_id) references estacionadas_motivos(id)
);

create table audit_log (
    id int identity,
    action varchar(10) not null,
    fecha datetime not null,
    access_ref int not null,
    entity_name varchar(20) not null,
    entity_data varchar(1000) not null,
    
    constraint pk_audit_log primary key (id),
    constraint fk_audit_log_usuarios foreign key (access_ref) references usuarios(access_def)
);

insert into usuarios(user_name, nombre, apellidos, central, supervisor) values
('user1', 'Usuario 1', 'Apellidos 1', 'S2', 0),
('user2', 'Usuario 2', 'Apellidos 2', 'MA', 0),
('user3', 'Usuario 3', 'Apellidos 3', 'MA', 0),
('sup1', 'Supervisor 1', 'Apellidos', 'MA', 1);

insert into tipologias(nombre) values 
('Tipologia 1'),
('Tipologia 2'),
('Tipologia 3'),
('Tipologia 4');

insert into permisos(nombre, access_ref) select 'modulo_estacionadas', access_def from usuarios where user_name in ('user1', 'user2', 'sup1');

insert into estacionadas_motivos(nombre) values
('Motivo 1'),
('Motivo 2'),
('Motivo 3');
